#include "gofbpmplot.h"
#include <cudata.h>
#include <QtDebug>
#include <QTime>
#include <culinkstats.h>
#include <cucontext.h>
#include <qwt_scale_widget.h>
#include <QPen>
#include <quplotdataconnector.h>
#include <quplotoptions.h>
#include <quaxiseventfilter.h>
#include <qwt_plot_grid.h>
#include <QMenu>

GofBpmPlot::GofBpmPlot(QWidget *parent, CumbiaPool *cu_pool, const CuControlsFactoryPool &fp, bool opengl)
    : QuArrayPlot(parent, opengl),  m_fftsiz(-1), m_freq(-1.0),
    m_regenerate_x(true), m_update_y_scale(true), m_period(100) {
    QwtPlotGrid * grid = static_cast<QwtPlotGrid *>(itemList(QwtPlotGrid::Rtti_PlotGrid).first());
    grid->setPen(QColor(Qt::lightGray), 0.0);
    QPalette p = axisWidget(QwtPlot::xBottom)->palette();
    p.setColor(QPalette::WindowText, Qt::darkGray);
    for(int id = 0; id < QwtPlot::axisCnt; id++) {
        axisWidget(id)->setPalette(p);
    }

    QuAxisEventFilter *af = new QuAxisEventFilter(this);
    connect(af, SIGNAL(doubleClicked()), this, SLOT(updateYScale()));
    connect(af, SIGNAL(contextMenuRequest()), this, SLOT(axisContextMenuRequest()));
    axisWidget(QwtPlot::yLeft)->installEventFilter(af);
    af->setProperty("axisId", QwtPlot::yLeft);
    m_conn = new QuPlotDataConnector(cu_pool, fp, this);
    connect(m_conn, SIGNAL(configured(QwtPlot*,CuData)),
            this, SLOT(configure(QwtPlot*,CuData)));
}

void GofBpmPlot::configure(QwtPlot *p, const CuData &da) {

}

void GofBpmPlot::addSource(const QString &src) {
    m_conn->addSource(src.toStdString());
}

void GofBpmPlot::unsetSources() {
    m_conn->unsetSources();
}

void GofBpmPlot::setOptions(const CuData& o) {
    m_conn->getContext()->setOptions(o);
    if(m_conn->getContext()->readers().size() > 0) // prevent link not initialized error msg
        m_conn->getContext()->sendData(o);
}

void GofBpmPlot::updateYScale() {
    m_update_y_scale = true;
}

void GofBpmPlot::setPeriod(int p) {
    m_period = p;
    setOptions(CuData(TTT::Period, p));
}

void GofBpmPlot::fftSizChanged(const CuData &v) {
    if(!v.B(CuDType::Err) && v.containsKey(CuDType::Value)) {
        long ffts = 1.0;
        v[CuDType::Value].to<long>(ffts);
        if(m_fftsiz != ffts) {
            m_fftsiz = ffts;
            m_regenerate_x = m_freq > 0; // dirty if m_freq is valid
        }
    }
}

void GofBpmPlot::fastFreqChanged(float f) {
    if(m_freq != f) {
        m_freq = f;
        m_regenerate_x = m_fftsiz > 0;
    }
}

void GofBpmPlot::axisContextMenuRequest() {
        QMenu m;
        QAction* a = m.addAction("Autoscale");
        a->setParent(&m);
        a->setCheckable(true);
        connect(a, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoscale(bool)));
        QuPlotOptions op(this);
        bool autoscale = axisAutoScale(sender()->property("axisId").toInt());
        a->setChecked(autoscale);
        a->setProperty("axisId", sender()->property("axisId").toInt());
        m.exec(QCursor::pos());
}

void GofBpmPlot::setAxisAutoscale(bool as)
{
    int axisId = sender()->property("axisId").toInt();
    if(axisId >= 0) {
        setAxisAutoScale(axisId, as);
    }
}

void GofBpmPlot::m_period_adjust(size_t siz) {
    if(siz > 100000 && m_period != siz / 100 ) {
        printf("Plot '%s': \e[1;35mslowing down period to %ld\e[0m\n", qstoc(objectName()), siz / 100);
        setPeriod(siz / 100);
    }
    else if(m_period != 100) {
        printf("Plot '%s': \e[1;35msetting period to 100ms\e[0m\n", qstoc(objectName()));
        setPeriod(100);
    }
}

void GofBpmPlot::m_y_autoscale(const char *src, const std::vector<double>& y) {
    double min = y.size() > 0 ? y[0] : 0;
    double max = min;
    for(size_t i = 0; i < y.size(); i++) {
        if(y[i] < min)
            min = y[i];
        if(y[i] > max)
            max = y[i];
    }
    pretty_pri("Y axis [%f, %f] this plot %p", min, max, this);
    if(min == max && min == 0) {
        min = -0.1; max = 0.1;    
    }
    QuPlotOptions o(this);
    o.ylo(min); // replots
    o.yup(max); // replots
    m_update_y_scale = false;
}



